from setuptools import setup

setup(name='snips_helper',
      version='0.5',
      description='Snips.ai Helper',
      url='http://github.com/uchagani/snips_helper',
      author='Umair Chagani',
      author_email='umair.chagani@gmail.com',
      license='MIT',
      packages=['snips_helper'],
      install_requires=['selenium'],
      zip_safe=False)